%{
/*******************************************************************************
* Copyright (c) 2007, Perforce Software, Inc.  All rights reserved.
* 
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
* 
* 1.  Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
* 
* 2.  Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE 
* SOFTWARE, INC. BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
* DAMAGE.
*******************************************************************************/

/*******************************************************************************
 *    Config file parser for p4dctl
 ******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "parsesupp.h"
#include "config.h"

#define YYDEBUG 1
extern char *yytext;

%}

%union {
    int			ival;
    char *		sval;
    void *		pval;
}

%type <sval>	var
%type <sval>	value
%type <sval>	word
%type <pval>	varspec
%type <pval>	varlist
%type <pval>	serverlist
%type <pval>	serverspec
%type <pval>	config


%token 	<sval> WORD STRING NUMBER PATH

%%

config:		varlist serverlist		{ AddEnviron( $1 ); }
      		| serverlist			{ $$ = $1;		     }
		|				{ }
		;

varlist:	varspec				{ $$ = $1; 		     }
       		| varlist varspec		{ $$ = AddVar( $1, $2 );     }
		;

varspec:	var '=' value	 		{ $$ = MakeVar( $1, $3 );    }
       		;

var:		word				{ $$ = $1;		     }
       		;

value:		word				{ $$ = $1;		     }
     		| STRING			{ $$ = strdup( yylval.sval );}
     		| PATH				{ $$ = strdup( yytext );     }
		| NUMBER			{ $$ = strdup( yytext );     }
		;

word:		WORD				{ $$ = strdup( yytext );     }
serverlist:	serverspec			{ $$ = $1;		     }
	  	| serverlist serverspec		
		;

serverspec:	word word '{' varlist '}'	{ $$ = MakeServer( $2, $1, $4);}
	  	;
